/*
 * norboot.c
 * Implementation of NOR boot loader
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*  AUTHOR	: Amit Chatterjee	                                      */
/*  DATE	: Sept-05-2005                                                */

#include <bootloader.h>

#ifdef UBL_USE_NOR_BOOT

#include <nor.h>
#include <error.h>
#include <armDsp.h>
#include <cslr_edma.h>

extern Uint32 gEntryPoint;
extern Uint32 gDMA;
extern void *ubl_alloc_mem(int size);

/* Function to find out where the Application is and copy to DRAM */
Uint32 NORCopy() {
	volatile NOR_BOOT	*hdr = 0;
	volatile Uint32		*appStartAddr = 0;
	volatile Uint32		count = 0;
	volatile Uint32		*ramPtr = 0;

	UARTSendData(21,(Uint8 *) "\r\nStarting NOR Copy\r\n");

	hdr = (volatile NOR_BOOT *)UBL_NOR_APP_START_ADDR;

	/* Magic number found */
	if((hdr->magicNum & 0xFFFFFF00) != MAGIC_NUMBER_VALID)
	{
	 	return E_FAIL;/* Magic number not found */
	}

	/* Set the global address */
	gEntryPoint = hdr->appStartAddr;

	/* Set the Start Address */
	appStartAddr = (Uint32 *)(UBL_NOR_APP_START_ADDR + NOR_BOOT_HDR_SIZE);

	if(hdr->magicNum == APP_MAGIC_BIN_IMG)
	{
		ramPtr = (Uint32 *) hdr->ldAddress;

		/* Copy data to RAM */
		for(count = 0; count < ((hdr->appSize + 3)/4); count ++)
		{
			ramPtr[count] = appStartAddr[count];
		}
		/* Since our entry point is set, just return success */
		return E_PASS;
	}

	if(SFMTDecode((Uint8 *)appStartAddr, hdr->appSize) != E_PASS)
	{
		return E_FAIL;
	}

 	return E_PASS;
}

#endif /* UBL_USE_NOR_BOOT */

