/*
 * uartmenu.c
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/** \file   uartmenu.c
    \brief  DaVinci Secondary boot-loader, UART Menu file

    NOTE: THIS FILE IS PROVIDED FOR INITIAL DEMO RELEASE AND MAY BE
          REMOVED AFTER THE DEMO OR THE CONTENTS OF THIS FILE ARE SUBJECT
          TO CHANGE.

    @author    Amit Chatterjee
    @version    0.1 - Created
**/

#include <bootloader.h>

#ifdef UBL_USE_UART_PROG

#include <uart.h>
#include <nor.h>
#include <nand.h>

/* External Varaibles */
extern Uint32 gAppStartAddr, gLoopCount, gMagicFlag;
extern Uint32	UARTGetSrecAddr (void);

#ifdef UBL_USE_NAND_BOOT
extern NAND_INFO	gNandInfo;
#endif

void UartProgrammer (void)
{
#ifdef UBL_USE_NAND_BOOT
	NAND_BOOT 	nandBoot;
#endif
#ifdef UBL_USE_NOR_BOOT
	Uint32		startAddr;
	NOR_BOOT	norBoot;
#endif
	Uint32		srecBuffer;

	/* Start the UART Download */
	UARTBoot();

	UARTSendData(24,(Uint8 *) "\r\nStarting Flash Write\r\n");

	switch (gMagicFlag)
	{
#ifdef UBL_USE_NOR_BOOT
		case APP_MAGIC_BURN_NOR:
		case APP_MAGIC_BURN_NOR_Z:
		{
			NORInit();

			startAddr = UBL_NOR_APP_START_ADDR;

			// Erasing the Flash
			UARTSendData(23, (Uint8 *)"Erasing the NOR Flash\r\n");

			EraseNORFlash(startAddr, gLoopCount);

			UARTSendData(17, (Uint8 *)"Erase Completed\r\n");

			srecBuffer = UARTGetSrecAddr();
			norBoot.magicNum = gMagicFlag;
			norBoot.appSize = gLoopCount;
			norBoot.appStartAddr = gAppStartAddr;
			norBoot.reserved = 0;

			/* Nor Burn */
			NORWriteNBytes((Uint8 *)startAddr, NOR_BOOT_HDR_SIZE, (Uint8 *) &norBoot);

			/* Advance the header size */
			startAddr += NOR_BOOT_HDR_SIZE;

			/* Nor Burn */
			NORWriteNBytes((Uint8 *)startAddr, gLoopCount, (Uint8 *) srecBuffer);
			break;
		}
#endif

#ifdef UBL_USE_NAND_BOOT
		case APP_MAGIC_BURN_NAND:
		case APP_MAGIC_BURN_NAND_Z:
		{
			/* Initialize Nand */
			NANDInit();

			srecBuffer = UARTGetSrecAddr();

			/* TODO - Check Logic */
			nandBoot.entryPoint	= gAppStartAddr;

			if(gNandInfo.page_size == 2112)
			{
				nandBoot.numPage	= gLoopCount/2048;
			}
			else
			{
				nandBoot.numPage	= gLoopCount/512;
			}

			nandBoot.block		= (gAppStartAddr >> gNandInfo.blk_shift);
			nandBoot.page		= (gAppStartAddr >> gNandInfo.page_shift) & (1<<gNandInfo.page_shift - 1);

			/* Nand Burn */
			NANDWrite(&nandBoot, (Uint8 *) srecBuffer);
		}
#endif
		case APP_MAGIC_SAFE:
			break;

		default:
			UARTSendData(19,(Uint8 *) "\r\nWrong BOOT Flag\r\n");
			return;
	}

	return;
}

#endif /* UBL_USE_UART_PROG */


