/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  AIC32 header file
 *
 */

#ifndef AIC32_
#define AIC32_

#ifdef __cplusplus
extern "C" {
#endif

#include "davincihd.h"
#include "davincihd_i2c.h"
#include "davincihd_mcasp.h"

/* ------------------------------------------------------------------------ *
 *  AIC32 ID                                                                *
 * ------------------------------------------------------------------------ */
#define AIC32_0                 0

/* ------------------------------------------------------------------------ *
 *  AIC32 I2C address                                                       *
 * ------------------------------------------------------------------------ */
#define AIC32_I2C_ADDR          0x18

/* ------------------------------------------------------------------------ *
 *  AIC32 Registers                                                         *
 * ------------------------------------------------------------------------ */
#define AIC32_NUMREGS           103
#define AIC32_PAGESELECT        0
#define AIC32_RESET             1

#define READONLY                0xaa00
#define RESERVED                0xffff

/* ------------------------------------------------------------------------ *
 *  AIC32 Data Structures                                                   *
 * ------------------------------------------------------------------------ */
typedef Int16 AIC32_Handle;

typedef struct {
    Uint16 regs[AIC32_NUMREGS];
} AIC32_Config;

/* ------------------------------------------------------------------------ *
 *  Prototypes                                                              *
 * ------------------------------------------------------------------------ */
/* Open/Close codec */
AIC32_Handle DAVINCIHD_AIC32_openCodec ( Uint32 id, AIC32_Config *config );
Int16        DAVINCIHD_AIC32_closeCodec( AIC32_Handle aic32 );

/* Get/Set registers */
Int16 DAVINCIHD_AIC32_rget     ( AIC32_Handle aic32, Uint16 regnum, Uint16* regval );
Int16 DAVINCIHD_AIC32_rset     ( AIC32_Handle aic32, Uint16 regnum, Uint16 regval );
Int16 DAVINCIHD_AIC32_rset_mask( AIC32_Handle aic32, Uint16 regnum, Uint16 regval, Uint8 mask );
Int16 DAVINCIHD_AIC32_config   ( AIC32_Handle aic32, AIC32_Config* config );

/* Read/Write 16-bit or 32-bit data */
Int16 DAVINCIHD_AIC32_read16   ( AIC32_Handle aic32, Int16* data16 );
Int16 DAVINCIHD_AIC32_read32   ( AIC32_Handle aic32, Int32* data32 );
Int16 DAVINCIHD_AIC32_write16  ( AIC32_Handle aic32, Int16 data16 );
Int16 DAVINCIHD_AIC32_write32  ( AIC32_Handle aic32, Int32 data32 );

/* Special functions */
void DAVINCIHD_AIC32_mute      ( AIC32_Handle aic32, Uint16 mode );

#ifdef __cplusplus
}
#endif

#endif
