/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  Board Setup
 *
 */

#include "davincihd.h"
#include "davincihd_i2c.h"
#include "davincihd_i2c_gpio.h"

/*
 *  For GEL startup code ( ARM only )
 */
#ifdef ARM_SIDE
    #include "davincihd_ddr.h"
    #include "davincihd_pll.h"
    #include "davincihd_psc.h"
#endif

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  DAVINCIHD_startupGEL( )                                                 *
 *      Setup for GEL file.                                                 *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_startupGEL( )
{
    #ifdef ARM_SIDE
        /* Setup Power Sleep Controller */
        DAVINCIHD_PSC_enableAll( );

        /* Setup PLL0 */
        DAVINCIHD_enablePll0( OSCIN, 21, 0 );   // Setup Pll1 ( DSP @ 594 MHz )

        /* Setup DDR2 */
      //DAVINCIHD_enablePll1( OSCIN, 21, 0 );   // Setup Pll2 ( DDR @ 297 MHz ) // Generates Content holes, Phantom Errors.
      //DAVINCIHD_DDR_init( 297 );              // Setup DDR2

      //DAVINCIHD_enablePll1( OSCIN, 20, 0 );   // Setup Pll2 ( DDR @ 283.5 MHz )
      //DAVINCIHD_DDR_init( 283 );              // Setup DDR2

        DAVINCIHD_enablePll1( OSCIN, 19, 0 );   // Setup Pll2 ( DDR @ 270 MHz )
        DAVINCIHD_DDR_init( 270 );              // Setup DDR2

    #endif
    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  DAVINCIHD_init( )                                                       *
 *      Setup PinMux                                                        *
 *      Setup I2C & I2C GPIO                                                *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_init( )
{
    /*
     *  Setup Pin Mux for board default
     */
    DAVINCIHD_setupPinMux( );

    /*
     *  PSC, PLL, DDR, & EMIFA are already configured in the GEL file
     *  However the function _startupGEL can be called to perform the
     *  same effect as the GEL file.
     */
    DAVINCIHD_startupGEL( );

    /*
     *  Setup I2C/I2C GPIO Expander
     */
    DAVINCIHD_I2C_init( );
    DAVINCIHD_I2C_GPIO_init( );
    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _setupPinMux( )                                                         *
 *                                                                          *
 *      Pin Mux can always be changed w/o any further setup, all that this  *
 *      function does is to set the initial state to a known state.         *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_setupPinMux( )
{
    Int16 pcien = SYS_PINMUX0 & ( 1 << 2 );

    if ( pcien == 0 )
    {
        SYS_PINMUX0 = 0
            | ( 0 << 31 )   // VBUSDIS      [USB_DRVVBUS]
            | ( 0 << 30 )   // STCCK        [GPIO4]
            | ( 0 << 29 )   // AUDCK1       [GPIO2]
            | ( 0 << 28 )   // AUDCK0       [GPIO3]
            | ( 0 << 24 )   // [2]CRGMUX    [no CRGEN]
            | ( 0 << 22 )   // [2]TSSOMUX   [VP_DOUT[11:8]]
            | ( 0 << 20 )   // [2]TSSIMUX   [VP_DIN[7:0]]
            | ( 0 << 18 )   // [2]PTSOMUX   [VP_DIN[7:0]]
            | ( 0 << 16 )   // [2]PTSIMUX   [VP_DIN[15:8]]
            | ( 0 << 5 )    // PINTD        [GPIO5]
            | ( 0 << 2 )    // PCIEN        [no PCI]
            | ( 0 << 1 )    // HPIEN        [no HPI]
            | ( 1 << 0 );   // ATAEN        [ATA/NAND]

        SYS_PINMUX1 = 0
            | ( 2 << 4 )    // [2]UART2CTL  [IrDA]
            | ( 3 << 2 )    // [2]UART1CTL  [GPIO]
            | ( 1 << 0 );   // [2]UART0CTL  [UART0 w/o Flow]
    }
    else
    {
        SYS_PINMUX0 = 0
            | ( 0 << 31 )   // VBUSDIS      [USB_DRVVBUS]
            | ( 0 << 30 )   // STCCK        [GPIO4]
            | ( 0 << 29 )   // AUDCK1       [GPIO2]
            | ( 0 << 28 )   // AUDCK0       [GPIO3]
            | ( 0 << 24 )   // [2]CRGMUX    [no CRGEN]
            | ( 0 << 22 )   // [2]TSSOMUX   [VP_DOUT[11:8]]
            | ( 0 << 20 )   // [2]TSSIMUX   [VP_DIN[7:0]]
            | ( 0 << 18 )   // [2]PTSOMUX   [VP_DIN[7:0]]
            | ( 0 << 16 )   // [2]PTSIMUX   [VP_DIN[15:8]]
            | ( 1 << 5 )    // PINTD        [PINTD]
            | ( 1 << 2 )    // PCIEN        [PCI]
            | ( 0 << 1 )    // HPIEN        [no HPI]
            | ( 0 << 0 );   // ATAEN        [no ATA]

        SYS_PINMUX1 = 0
            | ( 2 << 4 )    // [2]UART2CTL  [IrDA]
            | ( 3 << 2 )    // [2]UART1CTL  [GPIO]
            | ( 1 << 0 );   // [2]UART0CTL  [UART0 w/o Flow]
    }

    /* 3.3V Power Domain */
    SYS_VDD3P3VPWDN = 0
        | ( 1 << 28 )       // USBV     [Power Down][-> USB +5V]
        | ( 1 << 27 )       // CLKOUT   [Power Down][-> DC_P2]
        | ( 0 << 25 )       // SPI      [Power Up]
        | ( 0 << 24 )       // VLYNQ    [Power Up]
        | ( 0 << 21 )       // GMII     [Power Up]
        | ( 0 << 20 )       // MII      [Power Up]
        | ( 0 << 19 )       // McASP1   [Power Up]
        | ( 0 << 18 )       // McASP0   [Power Up]
        | ( 0 << 17 )       // PCIHPI1  [Power Up]
        | ( 0 << 16 )       // PCIHPI0  [Power Up]
        | ( 0 << 15 )       // GPIO     [Power Up]
        | ( 0 << 14 )       // WDTIM    [Power Up]
        | ( 0 << 13 )       // TIM23    [Power Up]
        | ( 0 << 12 )       // TIM01    [Power Up]
        | ( 0 << 11 )       // PWM1     [Power Up]
        | ( 0 << 10 )       // PWM0     [Power Up]
        | ( 0 << 9 )        // UR2FC    [Power Up]
        | ( 0 << 8 )        // UR2DAT   [Power Up]
        | ( 1 << 7 )        // UR1FC    [Power Down]
        | ( 1 << 6 )        // UR1DAT   [Power Down]
        | ( 0 << 5 )        // UR0MDM   [Power Up]
        | ( 0 << 4 )        // UR0DF    [Power Up]
        | ( 0 << 3 )        // VPIF3    [Power Up]
        | ( 0 << 2 )        // VPIF2    [Power Up]
        | ( 0 << 1 )        // VPIF1    [Power Up]
        | ( 0 << 0 );       // VPIF0    [Power Up]

    /* Aux Clock Controls */
    SYS_CLKCTL = 0
        | ( 0 << 25 )       // 24-MHz Power [PowerUp]
        | ( 0 << 24 )       // 24-MHz Osc   [Enable]
        | ( 0 << 16 )       // CLKOUT       [Disable]
        | ( 0 << 8 )        // AUDIO_CLK1   [Disable]
        | ( 0 << 0 );       // AUDIO_CLK0   [Disable]

    /* Video Clocks */
    SYS_VIDCLKCTL = 0
        | ( 4 << 12 )       // Video Chnl 3 Clock [VP_CLKIN0] // VID_CLK_B
        | ( 4 << 8 )        // Video Chnl 2 Clock [VP_CLKIN0] // VID_CLK_A
        | ( 1 << 0 );       // Video Chnl 1 Clock [VP_CLKIN1]

    /* TSIF ( Transport Stream Interface ) Clocks */
    SYS_TSIFCTL = 0
        | ( 0 << 12 )       // TSIF1 Clock Source [CRG1_VCXI]
        | ( 0 << 8 )        // TSIF1 Output Clock [CRG1_VCXI]
        | ( 0 << 4 )        // TSIF0 Clock Source [CRG0_VCXI]
        | ( 0 << 0 );       // TSIF0 Output Clock [CRG0_VCXI]

    /* VS Clock Disable */
    SYS_VSCLKDIS = 0
        | ( 1 << 11 )       // VPIF Ch 3    [Disable]
        | ( 1 << 10 )       // VPIF Ch 2    [Disable]
        | ( 1 << 9 )        // VPIF Ch 1    [Disable]
        | ( 1 << 8 )        // VPIF Ch 0    [Disable]
        | ( 1 << 7 )        // TSIF1 Count  [Disable]
        | ( 1 << 6 )        // TSIF0 Count  [Disable]
        | ( 1 << 5 )        // TSIF1 TX Clk [Disable]
        | ( 1 << 4 )        // TSIF0 TX Clk [Disable]
        | ( 1 << 3 )        // TSIF1 RX Clk [Disable]
        | ( 1 << 2 )        // TSIF0 RX Clk [Disable]
        | ( 1 << 1 )        // CRG 1 Enable [Disable]
        | ( 1 << 0 );       // CRG 0 Enable [Disable]

    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _setPinMux( pinmux0, pinmux1 )                                          *
 *      Set pin mux depending on the value of ( pinmux0 ) & ( pinmux1 )     *
 *      The 1's bit indicates which field to enable.                        *
 *                                                                          *
 *      Inputs:                                                             *
 *              pinmux0     -> PinMux Reg 0                                 *
 *              pinmux1     -> PinMux Reg 1                                 *
 *              vdd3p3vpwdn -> 3.3V power down register                     *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_setPinMux( Uint32 pinmux0, Uint32 pinmux1, Uint32 vdd3p3vpwdn )
{
    SYS_PINMUX0     |= pinmux0;
    SYS_PINMUX1     |= pinmux1;
    SYS_VDD3P3VPWDN |= vdd3p3vpwdn;
    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _clrPinMux( pinmux0, pinmux1 )                                          *
 *      Clear pin mux depending on the value of ( pinmux0 ) & ( pinmux1 )   *
 *      The 1's bit indicates which field to enable.                        *
 *                                                                          *
 *      Inputs:                                                             *
 *              pinmux0     -> PinMux Reg 0                                 *
 *              pinmux1     -> PinMux Reg 1                                 *
 *              vdd3p3vpwdn -> 3.3V power down register                     *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_clrPinMux( Uint32 pinmux0, Uint32 pinmux1, Uint32 vdd3p3vpwdn )
{
    SYS_PINMUX0     &= ~pinmux0;
    SYS_PINMUX1     &= ~pinmux1;
    SYS_VDD3P3VPWDN &= ~vdd3p3vpwdn;
    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _resetEMIF( chip_select )                                               *
 *      Reset EMIFA chip select timings to MAX                              *
 *                                                                          *
 *      Inputs:                                                             *
 *              chip_select -> EMIF_CS2/3/4/5                               *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_resetEMIF( Uint16 chip_select )
{
    switch ( chip_select )
    {
        case EMIF_CS2:
            EMIFA_A1CR |= EMIFA_MAX_TIMEOUT;    // CS2
            EMIFA_NANDFCR &= ~( 1 << 0 );       // NAND Flash [OFF]
            break;
        case EMIF_CS3:
            EMIFA_A2CR |= EMIFA_MAX_TIMEOUT;    // CS3
            EMIFA_NANDFCR &= ~( 1 << 1 );       // NAND Flash [OFF]
            break;
        case EMIF_CS4:
            EMIFA_A3CR |= EMIFA_MAX_TIMEOUT;    // CS4
            EMIFA_NANDFCR &= ~( 1 << 2 );       // NAND Flash [OFF]
            break;
        case EMIF_CS5:
            EMIFA_A4CR |= EMIFA_MAX_TIMEOUT;    // CS5
            EMIFA_NANDFCR &= ~( 1 << 3 );       // NAND Flash [OFF]
            break;
        default:
            return -1;
    }
    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _setupEMIF( chip_select, emif_timing, emif_mode  )                      *
 *      Set EMIFA chip select timings                                       *
 *                                                                          *
 *      Inputs:                                                             *
 *              chip_select -> EMIF_CS2/3/4/5                               *
 *              timing      -> Timing                                       *
 *              emif_mode   -> NAND_MODE or EMIF_MODE                       *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 DAVINCIHD_setupEMIF( Uint16 chip_select, Uint32 timing, Uint16 emif_mode )
{
    Uint32 nand_enable_bit;

    switch ( chip_select )
    {
        case EMIF_CS2:
            EMIFA_A1CR = timing;                // CS2
            break;
        case EMIF_CS3:
            EMIFA_A2CR = timing;                // CS3
            break;
        case EMIF_CS4:
            EMIFA_A3CR = timing;                // CS4
            break;
        case EMIF_CS5:
            EMIFA_A4CR = timing;                // CS5
            break;
        default:
            return -1;
    }

    nand_enable_bit = 1 << ( chip_select - EMIF_CS2 );

    switch( emif_mode )
    {
        case NAND_MODE:
            EMIFA_NANDFCR |= nand_enable_bit;   // NAND Flash [ON]
            break;

        case EMIF_MODE:
            EMIFA_NANDFCR &= ~nand_enable_bit;  // NAND Flash [OFF]
            break;
    }

    return 0;
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _wait( delay )                                                          *
 *      Wait in a software loop for 'x' delay                               *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_wait( Uint32 delay )
{
    volatile Uint32 i;
    for ( i = 0 ; i < delay ; i++ ){ };
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _waitusec( usec )                                                       *
 *      Wait in a software loop for 'x' microseconds                        *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_waitusec( Uint32 usec )
{
    #ifdef ARM_SIDE
        DAVINCIHD_wait( usec * 2 );
    #elif DSP_SIDE
        DAVINCIHD_wait( usec * 28 );
    #endif
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _waitmsec( msec )                                                       *
 *      Wait in a software loop for 'x' milliseconds                        *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_waitmsec( Uint32 msec )
{
    _waitusec( 1000 * msec );
}
