/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  NAND Flash interface
 *
 */

#include "davincihd_nandflash.h"

/*
 *  NAND Organization
 */
extern NAND_ORGANIZATION nand_org;

/*
 *  NAND Flash timeout
 */
extern Int32 nand_erase_timeout;

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _NAND_eraseBlock( start_block, block_count )                            *
 *      Erase the blocks of Flash memory                                    *
 *                                                                          *
 *      Inputs:                                                             *
 *          start_block = start block                                       *
 *          block_count = # of blocks                                       *
 *                                                                          *
 *      Return:                                                             *
 *          =0  pass                                                        *
 *          >0  failed - bad pages                                          *
 *          -1  timeout                                                     *
 *                                                                          *
 * ------------------------------------------------------------------------ */
Int16 _NAND_eraseBlock( Uint32 start_block, Uint32 block_count )
{
    Uint32 i;
    Uint32 block_addr;
    Uint32 status;
    Int16 retcode = 0;

    for ( i = start_block ; i < start_block + block_count ; i++ )
    {
    	/* Fix for large page addressing */
        block_addr = i * nand_org.block_addr_incr;

        /* Erase a single block */
        _NAND_CMD( CMD_ERASE );             // Erase block
        _NAND_BLOCK_ADDR( block_addr );     // Block Address
        _NAND_CMD( CMD_ERASE_CONFIRM );     // Confirm Erase

        if ( _NAND_busywait( nand_erase_timeout ) )
            return NAND_ERR_TIMEOUT;

		_waitmsec ( 3 );

        /* Check status of erase */
        _NAND_CMD( CMD_STATUS );            // Check Status
        status = NAND_DATA;

        if ( status & CMD_STATUS_ERROR )
        {
            retcode = -1;
        }
    }

    return retcode;
}
