/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 * 
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *  - Made compatible with flash utils.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  NAND Flash Test
 *
 */

#include "stdio.h"
#include "davincihd_nandflash.h"

/*
 *  NAND Flash Large Pages ( 2kb )
 */
static Uint8 rx[2048 + 64];
static Uint8 tx[2048 + 64];

#define APP_MAGIC_NUM		0xA1ACED66
#define ENTRY_OFFSET		0x81080000
#define LOAD_ADDRESS		ENTRY_OFFSET

/* ------------------------------------------------------------------------ *
 *  nandflash_test( )                                                       *
 * ------------------------------------------------------------------------ */

#define	MAX_IMAGE_SIZE	(0x1000000)
#pragma DATA_SECTION (nandImage, ".image")
Uint32 nandImage[MAX_IMAGE_SIZE];

#pragma DATA_SECTION (nandTestImage, ".testimage")
Uint32 nandTestImage[MAX_IMAGE_SIZE];

Int16 nandflash_test( )
{
	Uint32 nblocks;
	Uint32 npages;
	Uint32 nbytes;

	FILE	*fPtr;
	Uint8	*ramPtr;
	Int32	fileSize = 0;
	Int32	offset = 0;
	Int32	start_offset = 0;
	Int16	retcode = 0;
	Int8	fileName[256];
	Int8	answer[256];
	Uint8	gErase;
	Uint32	entryOffset = 0;
	Uint32  loadAddress = 0;
	Uint32	start_page;
	Uint32	num_of_pages;
	Uint32	app_start_page;
	Int32 	i, j;
	Uint32 	*p32;
	/* Errors */
	Int16	errors;
	Int32 	bad_pages = 0;
	Int32 	magic_num = 0;
	Uint32	count;
	Uint32	countMask;
	Uint32	next_block = 0;
	Uint32 	found[2];

	printf("Starting NAND Flash Writer\n", offset);
	printf("Note: First page of the block will be used to store boot header\n\n");


    /* Test Case */
    /* ---------------------------------------------------------------- *
     *  Initialize NAND Flash                                           *
     * ---------------------------------------------------------------- */
   DAVINCIHD_NANDFLASH_init();

    /* Determine the total number of pages */
    npages = DAVINCIHD_NANDFLASH_getTotalPages();

    if (npages == 0)
    {
	printf("	Error: NO pages found\n");
	return 1;
    } else {
	nblocks = npages /  NAND_PAGES_PER_BLOCK;
	nbytes = npages * NAND_PAGESIZE;
	printf("Total blocks: %d\n", nblocks);
	printf("Total pages:  %d\n", npages);
	printf("Total bytes:  %d\n", nbytes);
	}

	/* ---------------------------------------------------------------- *
     *  Read Application                                                *
	 * ---------------------------------------------------------------- */
	printf("Enter the File Name\n");
	scanf("%s", fileName);

	printf("Enter Block Offset \n NOTE: [1-5] for User BootLoader [6-50] for Application\n");
	scanf("%s", answer);

	fflush(stdin);

	printf("Do a global Erase \n");
	scanf("%c", &gErase);

	offset = strtol(answer, NULL, 16);

	/*[1-5] for User BootLoader,[6-50] for Application
		 all other block numbers are error*/
	if ((offset < 1) || (offset > 50)) {
		printf("Wrong Offset - Use 1 to 0x%x\n", 50);
		printf("Note values are treated as hex & offset	should be in multiples of %d Bytes\n",
				(NAND_PAGESIZE * NAND_PAGES_PER_BLOCK));
		return -1;
	}

	memset(answer, 0, 256);

	/* Check if we are writing the application or the boot loader */
	if (offset > 5)	{
		/*write the application*/
		printf("Enter application entry point\n");
		scanf("%s", answer);
		entryOffset = strtoul(answer, NULL, 16);

		fflush(stdin);
		printf("Enter application load Address\n");
		scanf("%s", answer);
		loadAddress = strtoul(answer, NULL, 16);

		start_page = offset * NAND_PAGES_PER_BLOCK ;
		app_start_page = start_page+1;
		magic_num = 0xA1ACED66;
	}
	else
	{
		/* write the bootloader */
		entryOffset = 0x100;
		start_page = offset * NAND_PAGES_PER_BLOCK ;
		app_start_page = start_page+1;
		magic_num = 0xA1ACED00;
		loadAddress = 0;
	}

	/* Clear the memory */
	memset(nandImage, 0, NAND_PAGESIZE);
	memset(tx, 0, NAND_PAGESIZE);

	/* Open an File from the hard drive*/
	fPtr = fopen(fileName, "rb");
	if(fPtr == NULL)
	{
		printf("File %s Open failed\n", fileName);
		return -1;
	}

	/* Initialize the pointer*/
	ramPtr = (Uint8 *) (&nandImage[NAND_PAGESIZE/4]);
	fileSize = 0;


	/* Read file size*/
	fseek(fPtr,0,SEEK_END);
	fileSize = ftell(fPtr);

	if(fileSize == 0)
	{
		printf("File read failed.. Closing APP\n");
		fclose (fPtr);
		return -1;
	}

	fseek(fPtr,0,SEEK_SET);

	if (offset < 5)	{
		/* Its a UBL - So ignore first 256 bytes */
		fread(ramPtr, 1, 256, fPtr);
		fileSize -= 0x100;
	}

	if (fileSize != fread(ramPtr, 1, fileSize, fPtr)) {
		printf("Warning !!! File Size mismatch\n");
	}

	fclose (fPtr);
	
   	/* ---------------------------------------------------------------- *
     *  Erase                                                         *  						     
	 * ---------------------------------------------------------------- */
	if (gErase == 'Y' || gErase == 'y') {
		printf( "     Erasing Flash [%d-%d] blocks\n", 0, nblocks );
		/* Cycle through NAND Flash Blocks */
		for (i = 0 ; i < 1024 ; i++) {
			retcode = NANDCheckBadBlock (i);
			if (retcode < 0)
			{
				printf ("Block No. %d is a bad block, skipping it\n", i);
				continue;
			}
			retcode = DAVINCIHD_NANDFLASH_erase(i, 1);
			if (retcode < 0)
			{
				printf ("Block No. %d is a bad block, marking it\n", i);
				NANDMarkBadBlock(i);
			}
		}
	} else {
		nblocks = (fileSize/(NAND_PAGESIZE * NAND_PAGES_PER_BLOCK)) + 1;
		printf( "     Erasing Flash [%d-%d] blocks\n", offset, (offset + nblocks - 1) );
		for (i = 1, j = 0 ; i < (nblocks + 1); i++) {
			retcode = NANDCheckBadBlock (offset + j);
			if (retcode < 0)
			{
				printf ("Block No. %d is a bad block, skipping it\n", (offset + j));
				offset++;
				i--;
				continue;
			}
			retcode = DAVINCIHD_NANDFLASH_erase(offset + j, 1);
			if (retcode < 0)
			{
				printf ("Block No. %d is a bad block, marking it\n", (offset + j));
				NANDMarkBadBlock(offset + j);
				offset++;
				i--;
			}
			found[j] = offset;
			j++;
		}
		offset = found[0];
	}

	if (retcode != 0) {
        printf("Error: Erasing code %d\n", retcode);
		return retcode | 0x8000;
	}

	/* Check for bad blocks */
NAND_Check_1:
	retcode = NANDCheckBadBlock (offset);
	if (retcode < 0)
	{
		printf ("Block No. %d is a bad block, skipping it\n", offset);
		offset++;

		if (offset > 50)
			return retcode;
		else
			goto NAND_Check_1;
	}
	start_page = offset * NAND_PAGES_PER_BLOCK ;
	app_start_page = start_page + 1;

    /* ---------------------------------------------------------------- *
	 *  Write                                                       *      						     
	 * ---------------------------------------------------------------- */
	printf("Writing Flash \n");
	num_of_pages = (fileSize/NAND_PAGESIZE) + 1;
	
	/* ---------------------------------------------------------------- *
	 * start writing the header page-page 1	
	 * ---------------------------------------------------------------- */
		
	count = 0;
	/* Iterate through all pages */
	for (i = start_page ; i < start_page + 1 ; i++)
	{
		/* Create write pattern */
		p32 = ( Uint32* )tx;
		*p32++ = magic_num; 	/*APP_MAGIC_NUM*/
		*p32++ = entryOffset; 	/*ENTRY_OFFSET*/
		*p32++ = num_of_pages;/*total number of pages*/
		*p32++ = offset; /*OFFSET*/
		/*Start page number from the offset*/
		*p32++ = 0x00000001;
		*p32++ = loadAddress;	/*LOAD_ADDRESS;*/

		/* Write to Flash */
		errors = DAVINCIHD_NANDFLASH_writePage( ( Uint32 )tx, offset, count, 1 );
	
		if (errors < 0)
		{
			printf ("Block No. %d is a bad block, marking it\n", offset);
			NANDMarkBadBlock(offset);
			break;
		}
	}
	if (errors < 0)
		goto NAND_Check_1;
	
	fPtr = fopen(fileName, "rb");
	fileSize = 0;

	
	/* ---------------------------------------------------------------- *
	 *	Write															*
	 * ---------------------------------------------------------------- */
	printf( "  Writing Flash [%d-%d] pages\n", app_start_page, (app_start_page + num_of_pages) - 1 );
	
	start_offset = offset;
	count = 1;
	countMask = NAND_PAGES_PER_BLOCK - 1;

	for ( i = app_start_page ; i < app_start_page + num_of_pages ; i++ )
	{
		/* Create write pattern */
		p32 = ( Uint32* )tx;
	
		fileSize = fread (p32, 1, NAND_PAGESIZE, fPtr);
	
		/* Write to Flash */
		if ((i / NAND_PAGES_PER_BLOCK) != start_offset && (next_block == 0))
		{
			offset++;
			count = 0;
			next_block = 1;
		}

NAND_Check_2:
		retcode = NANDCheckBadBlock (offset);
		if (retcode < 0)
		{
			printf ("Block No. %d is a bad block, skipping it\n", offset);
			offset++;

			if (offset > 50)
				return retcode;
			else
				goto NAND_Check_2;
		}

		errors = DAVINCIHD_NANDFLASH_writePage( ( Uint32 )tx, offset, (count & countMask), 1 );
	
		if (errors < 0)
		{
			printf ("Block No. %d is a bad block, marking it\n", offset);
			NANDMarkBadBlock(offset);
			break;
		}

		/* Read from Flash */
		errors = DAVINCIHD_NANDFLASH_readPage( ( Uint32 )rx, i, 1 );
	
		if (errors)
		{
			printf( "  Found bad page #%d: [%d]\n", ++bad_pages, i );
			return bad_pages;
		}
		count++;
	
	}

	if (errors < 0)
		goto NAND_Check_2;

	fclose (fPtr);
	
	bad_pages = 0;
	
	return 0;
}

