/*
 * Copyright 2007 by Spectrum Digital Incorporated.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*
 *  DaVinci HD Test Suite
 *
 */

#include "stdio.h"
#include "davincihd.h"

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  Testing Function                                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void TEST_execute( Int16 ( *funchandle )( ), char *testname, Int16 testid )
{
    Int16 status;

    /* Display test ID */
    printf( "%02d  Testing %s...\n", testid, testname );

    /* Call test function */
    status = funchandle( );

    /* Check for test fail */
    if ( status != 0 )
    {
        /* Print error message */
        printf( "     FAIL... error code %d... quitting\n", status );
    }
    else
    {
        /* Print error message */
        printf( "    PASS\n" );
    }
}

extern Int16 spirom_test();

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  main( )                                                                 *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void main( void )
{
    TEST_execute( spirom_test,  "spirom", 1 );

    printf( "\n***ALL Tests Passed***\n" );
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _wait( delay )                                                          *
 *      Wait in a software loop for 'x' delay                               *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_wait( Uint32 delay )
{
    volatile Uint32 i;
    for ( i = 0 ; i < delay ; i++ ){ };
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _waitusec( usec )                                                       *
 *      Wait in a software loop for 'x' microseconds                        *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_waitusec( Uint32 usec )
{
    #ifdef ARM_SIDE
        DAVINCIHD_wait( usec * 2 );
    #elif DSP_SIDE
        DAVINCIHD_wait( usec * 28 );
    #endif
}

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  _waitmsec( msec )                                                       *
 *      Wait in a software loop for 'x' milliseconds                        *
 *                                                                          *
 * ------------------------------------------------------------------------ */
void DAVINCIHD_waitmsec( Uint32 msec )
{
    _waitusec( 1000 * msec );
}
